/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d;

import java.io.*;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>Generic utility codes.
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class Utils {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(Utils.class);

    /**
     * Returns the Md5Hash of a given file.
     * http://www.javalobby.org/forums/thread.jspa?threadID=84420
     *
     * @param lastPath Path of the file
     * @return MD5Hash
     */
    public static String getFileMd5Hash(String lastPath) {
        // http://www.javalobby.org/forums/thread.jspa?threadID=84420
        InputStream is = null;
        String output = null;
        MessageDigest digest;

        try {
            digest = MessageDigest.getInstance("MD5");
            File f = new File(lastPath);
            is = new FileInputStream(f);
            byte[] buffer = new byte[8192];
            int read;
            try {
                while ((read = is.read(buffer)) > 0) {
                    digest.update(buffer, 0, read);
                }
                byte[] md5sum = digest.digest();
                BigInteger bigInt = new BigInteger(1, md5sum);
                output = bigInt.toString(16);
                logger.debug("MD5: " + output);
            } catch (IOException e) {
                logger.error("Unable to process file for MD5", e);
            } finally {
                try {
                    is.close();
                } catch (IOException e) {
                    logger.error("Unable to close input stream for MD5 calculation", e);
                }
            }
        } catch (FileNotFoundException ex) {
            logger.error(ex.toString());
        } catch (NoSuchAlgorithmException ex) {
            logger.debug(ex.toString());
        } finally {
            try {
                is.close();
            } catch (IOException ex) {
                logger.error(ex.toString());
            }
        }
        return output;
    }

    /**
     * Returns the Md5Hash of a given string.
     * http://www.javalobby.org/forums/thread.jspa?threadID=84420
     *
     * @param input String input to be encoded.
     * @return MD5Hash
     */
    public static String getStringMd5Hash(String input) {
        // http://www.javalobby.org/forums/thread.jspa?threadID=84420
        InputStream is = null;
        String output = null;
        MessageDigest digest;

        try {
            digest = MessageDigest.getInstance("MD5");

            digest.update(input.getBytes());

            byte[] md5sum = digest.digest();
            BigInteger bigInt = new BigInteger(1, md5sum);
            output = bigInt.toString(16);
            logger.debug("MD5: " + output);

        } catch (NoSuchAlgorithmException ex) {
            logger.debug(ex.toString());
        }
        return output;
    }
}
