/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d.agentCommunication;

import hu.sztaki.dsd.visionair.a2a3d.eventSources.RemoteTriggeredEventSource;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.eclipse.jetty.server.Connector;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.nio.SelectChannelConnector;
import org.eclipse.jetty.webapp.WebAppContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * The WebInterface is the communication layer between the agents.
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class WebInterface {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(WebInterface.class);
    /**
     * An http server (Jetty) for serving incoming requests
     */
    Server server;
    /**
     * The URL of the server.
     */
    String remoteURL;
    /**
     * The id of the agent.
     */
    static String sensorId;
    /**
     * The machineId identifying the sensor
     */
    String machineId;
    /**
     * The local Port of the sensor
     */
    private int localPort;
    /**
     * List of EventSource instances subscribed for remote notifications
     */
    private ArrayList<RemoteTriggeredEventSource> eventSourceQueue = new ArrayList<RemoteTriggeredEventSource>();

    /**
     * Instantiates a new WebInterface.
     *
     * @param xmlconfig Configuration parameters (see readme.txt)
     * @param localStore The local DataStore for unsent events
     */
    public WebInterface(SubnodeConfiguration xmlconfig) {
        try {
            // Create local Jetty server
            server = new Server();
            Connector connector = (Connector) new SelectChannelConnector();
            localPort = xmlconfig.getInt("LocalPort");
            connector.setPort(localPort);
            server.setConnectors(new Connector[]{connector});
            WebAppContext webapp = new WebAppContext();
            //webapp.setContextPath("/");

            //Try to find the root of the local http server
            String webAppHome;
            if ((webAppHome = xmlconfig.getString("WebAppHome")) == null) {
                logger.info("No WebAppHome was set in the configuration file");
                File file = new File(System.getProperty("user.dir") + System.getProperty("file.separator") + "RestEasy");
                if (file.exists() && file.isDirectory()) {
                    webAppHome = System.getProperty("user.dir") + System.getProperty("file.separator") + "RestEasy";
                }
                logger.info("WebAppHome found: " + webAppHome);
            } else {
                logger.info("WebAppHome was set in the configuration file: " + webAppHome);
            }
            if (!webAppHome.equals("")) {
                webapp.setWar(webAppHome);
                webapp.setDefaultsDescriptor(webAppHome + "/webdefault.xml");
                server.setHandler(webapp);
                logger.info("Starting the local server");
                server.start();
                //server.join();
            } else {
                logger.error("Fatal error. No WebAppHome found, the web admin interface cannot be instantiated.");
                System.exit(1);
            }

        } catch (Exception ex) {
            logger.error(ex.toString());
            System.exit(1);
        }
    }

    /**
     * Stops the running communication threads. Called in the ShutdownHook of
     * the A2A3D.
     */
    public void close() {
        try {
            logger.debug("STOPPING");
            server.stop();
            logger.debug("DESTROYING");
            server.destroy();
        } catch (Exception ex) {
            logger.error(ex.toString());
        }
    }

    public void subscribeForAgentMessages(RemoteTriggeredEventSource remoteTriggeredEventSource) {
        eventSourceQueue.add(remoteTriggeredEventSource);
    }

    public Iterator<RemoteTriggeredEventSource> subscribedForAgentMessagesIterator() {
        Iterator<RemoteTriggeredEventSource> iterator = eventSourceQueue.iterator();
        return iterator;
    }

    public static String getSensorId() {
        return sensorId;
    }
}
