/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d.agentCommunication.resource;

import hu.sztaki.dsd.visionair.a2a3d.A2A3D;
import hu.sztaki.dsd.visionair.a2a3d.agentCommunication.AgentResponseString;
import hu.sztaki.dsd.visionair.a2a3d.eventSources.RemoteTriggeredEventSource;
import java.util.Iterator;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * <p>Communication interface to the agent for collecting remote events.
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
@Path("/remoteEvent")
@Produces("application/json;charset=UTF-8")
public class RemoteEvent {
    @Context
    private HttpServletRequest request;
    
    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(RemoteEvent.class);

    /**
     * Returns a test resource to validate the RestEasy installation.
     *
     * @param msg The incoming agent message.
     * @param response HttpServletResponse response.
     * @return AgentResponseString
     */
    @POST
    @Path("/")
    @Consumes("application/x-www-form-urlencoded")
    public AgentResponseString incomingEventHandle(
            @QueryParam("msg") String msg,
            @QueryParam("event") String eventString,
            @Context HttpServletResponse response) {
        
        Map requests = request.getParameterMap();
        
        logger.debug("incomingEventHandle is running");
        logger.debug("msg:" + msg);
        logger.debug("eventString:" + eventString);
        
        Iterator<RemoteTriggeredEventSource> esIterator = A2A3D.instance.webInterface.subscribedForAgentMessagesIterator();
        while (esIterator.hasNext()) {
            RemoteTriggeredEventSource rtes = esIterator.next();
            rtes.remoteMessage(msg);
        }
        AgentResponseString retval = new AgentResponseString();
        retval.content = "OK";
        return retval;
    }
}
