/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.A2A3D;
import hu.sztaki.dsd.visionair.a2a3d.Event;
import hu.sztaki.dsd.visionair.a2a3d.localStorage.DataStore;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt;
 * (MTA SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class DbWriteEventSourceImpl extends SimpleEventSource {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(DbWriteEventSourceImpl.class);
    /**
     * Configuration parameters set at instantiation time.
     */
    private SubnodeConfiguration xmlConfig;

    private DataStore localStore = null;

    /**
     * Instantiates a new DbWriteEventSourceImpl.
     *
     * @param xmlConfig Configuration parameters.
     */
    public DbWriteEventSourceImpl(SubnodeConfiguration xmlConfig) {
        this.xmlConfig = xmlConfig;
    }

    @Override
    public void init() {
        setStatus(STATUS_INITIATING);
        localStore = A2A3D.getInstance().getDataStore();
        super.init();
    }

    /**
     * Stores the incoming <code>Event</code> in the persistent database.
     *
     * @param event the Event to be stored in the persistent database.
     */
    @Override
    public boolean onHookEvent(Event event) {
        if (!super.onHookEvent(event)) {
            // Checks the event filters
            return false;
        }
        logger.info("Storing: " + event);
        localStore.storeEvent(event);
        return true;
    }
}
