/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.apache.http.HttpHost;
import org.apache.http.NameValuePair;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt;
 * (MTA SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
class HttpPostEventRouterImpl extends SimpleEventSource {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(HttpPostEventRouterImpl.class);
    /**
     * Proxy parameter to access the server: URL of the proxy
     */
    private static String proxyHost;
    /**
     * Proxy parameter to access the server: Port of the proxy
     */
    private static int proxyPort;
    /**
     * Proxy parameter to access the server: Username to the proxy
     */
    private static String proxyUser;
    /**
     * Proxy parameter to access the server: Password to the proxy
     */
    private static String proxyPw;
    /**
     * The URL of the remote consumer
     */
    String remoteURL;
    DefaultHttpClient httpclient;

    private long messagesSent = 0;

    public HttpPostEventRouterImpl(SubnodeConfiguration xmlConfig) {
        if (xmlConfig.containsKey("Parameters.ProxyHost")) {
            proxyHost = xmlConfig.getString("Parameters.ProxyHost");
        }
        if (xmlConfig.containsKey("Parameters.ProxyPort")) {
            proxyPort = xmlConfig.getInt("Parameters.ProxyPort");
        }
        if (xmlConfig.containsKey("Parameters.ProxyUser")) {
            proxyUser = xmlConfig.getString("Parameters.ProxyUser");
        }
        if (xmlConfig.containsKey("Parameters.ProxyPw")) {
            proxyPw = xmlConfig.getString("Parameters.ProxyPw");
        }

        remoteURL = xmlConfig.getString("Parameters.RemoteURL");

    }

    @Override
    public void init() {
        setStatus(STATUS_INITIATING);
        httpclient = new DefaultHttpClient();

        if (proxyHost != null) {
            HttpHost proxy = new HttpHost(proxyHost, proxyPort);
            httpclient.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY, proxy);
            if (proxyUser != null) {
                httpclient.getCredentialsProvider().setCredentials(
                        new AuthScope("localhost", 8080),
                        new UsernamePasswordCredentials("username", "password"));

            }
        }
        super.init();
    }

    /**
     * It makes the SimpleEventRouterImpl more general and easier to override to
     * separate the route and routeSend functions. This way other EventRouters
     * which needs the filtering facility might simply extend the
     * SimpleEventRouterImpl and include their features in the routeSend()
     * function.
     *
     * @param event
     */
    @Override
    public boolean onHookEvent(Event event) {
        logger.debug("onHookEvent");
        if (!super.onHookEvent(event)) {
            // Checks the event filters
            return false;
        }
        try {
            logger.debug("POST event started (" + remoteURL + ")");
            List<NameValuePair> formparams = new ArrayList<NameValuePair>();

            ObjectMapper mapper = new ObjectMapper();
            StringWriter sw = new StringWriter();
            mapper.writeValue(sw, event);
            sw.flush();

            formparams.add(new BasicNameValuePair("event", sw.toString()));
            UrlEncodedFormEntity entity = new UrlEncodedFormEntity(formparams, "UTF-8");
            HttpPost httpPost = new HttpPost(remoteURL);
            httpPost.setEntity(entity);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();
            String responseBody = httpclient.execute(httpPost, responseHandler);
            messagesSent++;
            logger.debug("RESPONSE FOR POST: " + responseBody);
            return true;
        } catch (ClientProtocolException ex) {
            logger.error(ex.toString());
        } catch (IOException ex) {
            logger.error(ex.toString());
        }
        return false;
    }

    public long getMessagesSent() {
        return messagesSent;
    }
}
