/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import java.io.File;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.apache.commons.io.input.Tailer;
import org.apache.commons.io.input.TailerListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt;
 * (MTA SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class LogTailEventSourceImpl extends SimpleEventSource implements TailerListener {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(LogTailEventSourceImpl.class);
    
    private int delay;
    
    private Tailer tailer;
    
    private Thread thread;

    /**
     * Instantiates a new DbWriteEventSourceImpl.
     *
     * @param xmlConfig Configuration parameters.
     */
    public LogTailEventSourceImpl(SubnodeConfiguration xmlConfig) {
        super(xmlConfig);
    }
    
    @Override
    public void init() {
        setStatus(STATUS_INITIATING);
        String filenameToTail = this.getConfiguration().getString("Parameters.Filename");
        logger.debug("Monitoring: " + filenameToTail);
        delay = this.getConfiguration().getInt("Parameters.Delay", 1000);
        logger.debug("Monitoring interval: " + delay);
        
        if (filenameToTail == null) {
            logger.error("Missing filename for monitored log file");
            setStatus(STATUS_ERROR);
            return;
        }
        
        File fileToTail = new File(filenameToTail);
        this.tailer = new Tailer(fileToTail, this, delay, true);
        this.thread = new Thread(tailer);
        thread.start();
        
        if (!getStatus().equals(STATUS_ERROR)) {
            setStatus(STATUS_RUNNING);
        }
    }

    /**
     * Needed because we are implementing the TailerListener interface. 
     * @param tailer 
     */
    @Override
    public void init(Tailer tailer) {
    }
    
    @Override
    public void fileNotFound() {
        logger.error("Log file to monitor not found");
        setStatus(STATUS_ERROR);
    }
    
    @Override
    public void fileRotated() {
        // Not monitored, but implemented because of the applied interface
        logger.debug("File rotated");
    }
    
    @Override
    public void handle(String string) {
        if (!getStatus().equals(STATUS_STOPPING)) {
            setStatus(STATUS_RUNNING);
        }
        Event event = new Event("", Event.EVENT_TYPE_TAIL_CHANGE, string);
        
        logger.debug("Handle: " + string);
        this.notifyQueue(event);
    }
    
    @Override
    public void handle(Exception excptn) {
        setStatus(STATUS_ERROR);
    }
    
    public void destroy() {
        setStatus(STATUS_STOPPING);
        this.tailer.stop();
        while (this.thread.isAlive()) {
            try {
                this.thread.join();
            } catch (InterruptedException ex) {
            }
        }
        logger.debug("Tailer thread stopped");
        super.destroy();
    }
    
}
