/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import java.util.HashMap;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>SimpleTimedEvent event source factory. Instantiates QuartzSimpleTimedEventSource
 * EventSources.
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class SimpleTimedEvent implements EventSourceFactory {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(SimpleTimedEvent.class);
    /** HashMap of QuartzSimpleTimedEventSource EventSource instances. */
    private static HashMap<String, QuartzSimpleTimedEventSource> instances = null;

    /**
     * Instantiates a new SimpleTimedEvent.
     */
    public SimpleTimedEvent() {
        if (instances == null) {
            logger.debug("New Hashmap created for SimpleTimedEvent");
            instances = new HashMap<String, QuartzSimpleTimedEventSource>();
        }
    }

    /**
     * Returns a running instance of QuartzSimpleTimedEventSource. If a QuartzSimpleTimedEventSource with
     * the id passed in xmlConfig has already been instantiated, then that instance
     * will be returned, otherwise a new is created.
     *
     * @param xmlConfig xmlConfigConfiguration parameters to the WindowsFileSystemWatcherImpl.
     * @return QuartzSimpleTimedEventSource
     * @throws hu.sztaki.dsd.visionair.a2a3d.eventSources.NoEventSourceAviableError
     */
    @Override
    public EventSource getInstance(SubnodeConfiguration xmlConfig) throws NoEventSourceAviableError {
        if (instances.containsKey(xmlConfig.getString("Id"))) {
            return (QuartzSimpleTimedEventSource) instances.get(xmlConfig.getString("Id"));
        }
        QuartzSimpleTimedEventSource instance = new QuartzSimpleTimedEventSource(xmlConfig);
        instances.put(xmlConfig.getString("Id"), instance);
        return instance;
    }

    /**
     * Returns a running instance of QuartzSimpleTimedEventSource with the given id.
     *
     * @param id An id identifying the EventSource among the QuartzSimpleTimedEventSource instances.
     * @return QuartzSimpleTimedEventSource
     * @throws hu.sztaki.dsd.visionair.a2a3d.eventSources.NoEventSourceAviableError
     */
    public EventSource getInstance(String id) throws NoEventSourceAviableError {
        return (EventSource) instances.get(id);
    }
}
