/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
class UserAlertEventSourceImpl extends SimpleEventSource {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(UserAlertEventSourceImpl.class);
    private int closeWindowTimeout = 0;

    public UserAlertEventSourceImpl() {
        super();
    }
    
    public UserAlertEventSourceImpl(SubnodeConfiguration xmlConfig) {
        super(xmlConfig);
    }
    
    @Override
    public void init() {
        setStatus(STATUS_INITIATING);
        SubnodeConfiguration xmlConfig = getConfiguration();

        if (xmlConfig!= null && xmlConfig.containsKey("Parameters.CloseWindowTimeout")) {
            logger.debug("CloseWindowTimeout: " + xmlConfig.getInt("Parameters.CloseWindowTimeout"));
            this.closeWindowTimeout = xmlConfig.getInt("Parameters.CloseWindowTimeout") * 1000;
        }
        super.init();
    }

    /**
     * It makes the SimpleEventRouterImpl more general and easier to override to
     * separate the route and routeSend functions. This way other EventRouters
     * which needs the filtering facility might simply extend the
     * SimpleEventRouterImpl and include their features in the routeSend()
     * function.
     *
     * @param collectedEvent
     */
    @Override
    public boolean onHookEvent(Event event) {
        if (!super.onHookEvent(event)) {
            // Checks the event filters
            return false;
        }
        Thread alertThread = new Thread(new MessageNotificator(event, this.closeWindowTimeout));
        alertThread.start();
        return true;
    }

    /**
     * The notifications of the DataCollectors happen in separate Threads. These
     * threads run the ThreadNotificators which handle the actual notification.
     */
    private class MessageNotificator implements Runnable {

        Event collectedEvent;
        int timeout;

        /**
         * Instantiates a new MessageNotificator.
         *
         * @param event TriggerMessage to be sent to the DataCollector.
         */
        public MessageNotificator(Event collectedEvent, int timeout) {
            this.collectedEvent = collectedEvent;
            this.timeout = timeout;
        }

        /**
         * Triggers the DataCollector with the TriggerMessage.
         */
        @Override
        public void run() {
            logger.debug("Alerting: " + collectedEvent);

            JOptionPane pane = new JOptionPane(
                    "Incoming event:\n\n" + collectedEvent);
            Object[] options = new String[]{"Ok"};
            pane.setOptions(options);
            JDialog dialog = pane.createDialog(new JFrame(), "Dialog");
            if (this.timeout > 0) {
                Thread stopThread = new Thread(new MessageNotificatorStop(dialog, this.timeout));
                stopThread.start();
            }
            dialog.setVisible(true);



            dialog.toFront();

            Object obj = pane.getValue();
            /*
             * int result = -1; for (int k = 0; k < options.length; k++) { if
             * (options[k].equals(obj)) { result = k; } }
             */
            logger.debug("User hit OK or timeout");
        }
    }

    private class MessageNotificatorStop implements Runnable {

        JDialog dialog = null;
        int timeout = 0;

        public MessageNotificatorStop(JDialog dialog, int closeWindowTimeout) {
            this.dialog = dialog;
            this.timeout = closeWindowTimeout;
        }

        @Override
        public void run() {
            try {
                //logger.debug("Thread.sleep start: "+this.timeout);
                Thread.sleep(this.timeout); // will sleep for 5 seconds
                //logger.debug("Thread.sleep stop");
            } catch (InterruptedException ex) {
                logger.debug(ex.toString());
            }
            logger.debug("StopThread - run");
            if (dialog != null && dialog.isVisible()) {
                dialog.setVisible(false);
            }
        }
    }
}
