/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package hu.sztaki.dsd.visionair.a2a3d.eventSources.eventFilters;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import java.util.List;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Denies or accepts events based on their types.
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt;
 * (MTA SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class EventTypeEventFilterImpl extends SimpleEventFilterImpl {

    /**
     * Logger of the class.
     */
    private static Logger logger = LoggerFactory.getLogger(EventTypeEventFilterImpl.class);
    private List<String> acceptedEventTypes = null;
    private List<String> deniedEventTypes = null;

    @Override
    public int check(Event event) {
        logger.debug("CHECKING **************************************************");
        if (acceptedEventTypes.contains(event.getEventType())) {
            logger.debug("CHECKING ************************************************** ACCEPT");
            return ACCEPT;
        }
        if (deniedEventTypes.contains(event.getEventType())) {
            logger.debug("CHECKING ************************************************** DENY");
            return DENY;
        }
        return NONE;
    }

    EventTypeEventFilterImpl(SubnodeConfiguration configurationAt) {
        @SuppressWarnings("unchecked")
        List<String> acceptedEventTypesTemp = configurationAt.getList("Accept.Type");
        acceptedEventTypes = acceptedEventTypesTemp;
        @SuppressWarnings("unchecked")
        List<String> deniedEventTypesTemp = configurationAt.getList("Deny.Type");
        deniedEventTypes = deniedEventTypesTemp;
    }
}
