/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.apache.commons.configuration.XMLConfiguration;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt;
 * (MTA SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class HttpPostEventRouterImplTest {

    public HttpPostEventRouterImplTest() {
    }

    @BeforeClass
    public static void setUpClass() {
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of init method, of class HttpPostEventRouterImpl.
     */
    @Test
    public void testInit() {
        System.out.println("init");

        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString
                = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.HttpPostEventRouter</ClassName>\n"
                + "            <Id>HttpPostEventRouter1</Id>\n"
                + "            <Description>Sends the events to a remote http url via Post method as JSON objects</Description>\n"
                + "            <RemoteURL>http://joker.dsd.sztaki.hu/visionair/agentCommPostTest.php</RemoteURL>\n"
                + "        </EventSource>";
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        final HttpPostEventRouterImpl instance = new HttpPostEventRouterImpl(sconfig);

        instance.init();
        instance.destroy();
    }

    /**
     * Test of onHookEvent method, of class HttpPostEventRouterImpl.
     */
    @Test
    public void testOnHookEvent() {
        System.out.println("onHookEvent");
        Event collectedEvent = null;
        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString
                = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.HttpPostEventRouter</ClassName>\n"
                + "            <Id>HttpPostEventRouter1</Id>\n"
                + "            <description>Sends the events to a remote http url via Post method as JSON objects</description>\n"
                + "            <RemoteURL>http://joker.dsd.sztaki.hu/visionair/agentCommPostTest.php</RemoteURL>\n"
                + "        </EventSource>";
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        final HttpPostEventRouterImpl instance = new HttpPostEventRouterImpl(sconfig);

        instance.init();
        instance.onHookEvent(collectedEvent);
        instance.onHookEvent(collectedEvent);
        instance.destroy();
    }

    /**
     * Test of onHookEvent method, of class HttpPostEventRouterImpl.
     */
    @Test
    public void testOnHookEventPerformance1() {
        System.out.println("onHookEvent");
        Event collectedEvent = null;
        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.HttpPostEventRouter</ClassName>\n"
                + "            <Id>HttpPostEventRouter1</Id>\n"
                + "            <description>Sends the events to a remote http url via Post method as JSON objects</description>\n"
                + "            <RemoteURL>http://joker.dsd.sztaki.hu/visionair/agentCommPostTest.php</RemoteURL>\n"
                + "        </EventSource>";
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        final HttpPostEventRouterImpl instance = new HttpPostEventRouterImpl(sconfig);

        instance.init();
        for (int i = 0; i < 1000; i++) {
            instance.onHookEvent(collectedEvent);
        }
        System.out.println("Finishedsending: " + instance.getMessagesSent());
        instance.destroy();
    }
}
