/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;
import mockit.FullVerifications;
import mockit.Injectable;
import mockit.Mock;
import mockit.MockUp;
import mockit.Mocked;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.AfterClass;
import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt;
 * (MTA SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class LogTailEventSourceImplTest {

    public LogTailEventSourceImplTest() {
    }

    @BeforeClass
    public static void setUpClass() {
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of init method, of class LogTailEventSourceImpl.
     */
    @Test
    public void testInit_0args() {
        System.out.println("init - Test missing filename during the init");

        // Test missing filename during the init
        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.LogTailEventSource</ClassName>\n"
                + "            <Id>LogTailEventSource1</Id>\n"
                + "            <Description>Java implementation of the UNIX tail -f command</Description>\n"
                + "            <Parameters>\n"
                + "                <Delay>1</Delay>\n"
                + "            </Parameters>\n"
                + "        </EventSource>";
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        LogTailEventSourceImpl instance = new LogTailEventSourceImpl(sconfig);
        assertEquals(EventSource.STATUS_NONE, instance.getStatus());
        instance.init();
        assertEquals(EventSource.STATUS_ERROR, instance.getStatus()); // No filename added

        System.out.println("init - Test bad filename during the init");
        xmlConfig = new XMLConfiguration();
        xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.LogTailEventSource</ClassName>\n"
                + "            <Id>LogTailEventSource1</Id>\n"
                + "            <Description>Java implementation of the UNIX tail -f command</Description>\n"
                + "            <Parameters>\n"
                + "                <Filename>bad_filename.log</Filename>\n"
                + "                <Delay>100</Delay>\n"
                + "            </Parameters>\n"
                + "        </EventSource>";
        stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        instance = new LogTailEventSourceImpl(sconfig);
        assertEquals(EventSource.STATUS_NONE, instance.getStatus());
        instance.init();
        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertEquals(EventSource.STATUS_ERROR, instance.getStatus()); // No filename added
        instance.destroy();

        System.out.println("init - Test good filename during the init");

        File temp = null;
        try {

            //create a temp file
            temp = File.createTempFile("good_filename", ".log");

            System.out.println("Temp file : " + temp.getAbsolutePath());

        } catch (IOException e) {

            e.printStackTrace();

        }

        xmlConfig = new XMLConfiguration();
        xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.LogTailEventSource</ClassName>\n"
                + "            <Id>LogTailEventSource1</Id>\n"
                + "            <Description>Java implementation of the UNIX tail -f command</Description>\n"
                + "            <Parameters>\n"
                + "                <Filename>" + temp.getAbsolutePath() + "</Filename>\n"
                + "                <Delay>100</Delay>\n"
                + "            </Parameters>\n"
                + "        </EventSource>";
        stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        instance = new LogTailEventSourceImpl(sconfig);
        assertEquals(EventSource.STATUS_NONE, instance.getStatus());

        instance.init();
        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertEquals(EventSource.STATUS_RUNNING, instance.getStatus());
        instance.destroy();
        temp.delete();

    }

    /**
     * Test of init method, of class LogTailEventSourceImpl.
     */
    @Test
    public void testInit_Tailer() {
        System.out.println("init with Tailer");
        // Nothing is done there. If the other init works well, this implementation
        // is good as well.
    }

    /**
     * Test of fileNotFound method, of class LogTailEventSourceImpl.
     */
    @Test
    public void testFileNotFound() {
        System.out.println("fileNotFound");
        // Tested in testInit_0args
    }

    /**
     * Test of fileRotated method, of class LogTailEventSourceImpl.
     */
    @Test
    public void testFileRotated() {
        System.out.println("fileRotated");
        // Nothing to do here.
    }

    /**
     * Test of handle method, of class LogTailEventSourceImpl.
     */
    @Test
    public void testHandle_String(@Mocked final Event mockedEvent) {
        System.out.println("handle");
        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.LogTailEventSource</ClassName>\n"
                + "            <Id>LogTailEventSource1</Id>\n"
                + "            <Description>Java implementation of the UNIX tail -f command</Description>\n"
                + "            <Parameters>\n"
                + "                <Filename>bad_filename.log</Filename>\n"
                + "                <Delay>100</Delay>\n"
                + "            </Parameters>\n"
                + "        </EventSource>";
        ByteArrayInputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        LogTailEventSourceImpl instance = new LogTailEventSourceImpl(sconfig);
        instance.handle("asd");

        new FullVerifications() {
            {
                new Event("", Event.EVENT_TYPE_TAIL_CHANGE, "asd");
                times = 1;
            }
        };
    }

    /**
     * Test of handle method, of class LogTailEventSourceImpl.
     */
    @Test
    public void testHandle_Exception() {
        System.out.println("handle");
        // Not tested
    }

    /**
     * Test of destroy method, of class LogTailEventSourceImpl.
     */
    @Test
    public void testDestroy() {
        System.out.println("destroy");
        // Tested in testHandleCalled()
    }

    /**
     * Test if the handle method called properly.
     */
    @Test
    public void testHandleCalled(@Injectable
            final SimpleEventSource mockedEventSource) {
        System.out.println("Test if the handle method called properly");
        File temp = null;
        try {

            //create a temp file
            temp = File.createTempFile("good_filename", ".log");

            System.out.println("Temp file : " + temp.getAbsolutePath());

        } catch (IOException e) {
            e.printStackTrace();
        }

        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.LogTailEventSource</ClassName>\n"
                + "            <Id>LogTailEventSource1</Id>\n"
                + "            <Description>Java implementation of the UNIX tail -f command</Description>\n"
                + "            <Parameters>\n"
                + "                <Filename>" + temp.getAbsolutePath() + "</Filename>\n"
                + "                <Delay>100</Delay>\n"
                + "            </Parameters>\n"
                + "        </EventSource>";
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        LogTailEventSourceImpl instance = new LogTailEventSourceImpl(sconfig);
        assertEquals(EventSource.STATUS_NONE, instance.getStatus());

        instance.init();
        instance.addToQueue("1", mockedEventSource);

        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertEquals(EventSource.STATUS_RUNNING, instance.getStatus());
        String testContent = "qwe\n";
        try {
            System.out.println("Write: " + testContent);
            FileUtils.writeStringToFile(temp, testContent, true);

        } catch (IOException ex) {
            fail("IOException");
        }

        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        testContent = "rty\n";
        try {
            System.out.println("Write: " + testContent);
            FileUtils.writeStringToFile(temp, testContent, true);

        } catch (IOException ex) {
            fail("IOException");
        }
        try {
            Thread.sleep(500);
        } catch (InterruptedException ex) {
        }

        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(withInstanceOf(Event.class));
                times = 2;
            }
        };

        instance.destroy();
        testContent = "uio\n";
        try {
            System.out.println("Write: " + testContent);
            FileUtils.writeStringToFile(temp, testContent, true);

        } catch (IOException ex) {
            fail("IOException");
        }
        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }

        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(withInstanceOf(Event.class));
                times = 2;
            }
        };
        temp.delete();
    }

    /**
     * Test if the content of the resulting Event is proper
     */
    @Test
    public void testHandleResultEvent(@Mocked
            final Event mockedEvent) {
        System.out.println("Test if the content of the resulting Event is proper");

        final SimpleEventSource mockedEventSource = new SimpleEventSource();

        File temp = null;
        try {

            //create a temp file
            temp = File.createTempFile("good_filename", ".log");

            System.out.println("Temp file : " + temp.getAbsolutePath());

        } catch (IOException e) {
            e.printStackTrace();
        }

        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.LogTailEventSource</ClassName>\n"
                + "            <Id>LogTailEventSource1</Id>\n"
                + "            <Description>Java implementation of the UNIX tail -f command</Description>\n"
                + "            <Parameters>\n"
                + "                <Filename>" + temp.getAbsolutePath() + "</Filename>\n"
                + "                <Delay>100</Delay>\n"
                + "            </Parameters>\n"
                + "        </EventSource>";
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());
        LogTailEventSourceImpl instance = new LogTailEventSourceImpl(sconfig);
        assertEquals(EventSource.STATUS_NONE, instance.getStatus());

        instance.init();
        instance.addToQueue("1", mockedEventSource);

        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertEquals(EventSource.STATUS_RUNNING, instance.getStatus());
        String testContent = "qwe\n";
        try {
            System.out.println("Write: " + testContent);
            FileUtils.writeStringToFile(temp, testContent, true);

        } catch (IOException ex) {
            fail("IOException");
        }

        try {
            Thread.sleep(200);
        } catch (InterruptedException ex) {
        }

        new FullVerifications() {
            {
                new Event("", Event.EVENT_TYPE_TAIL_CHANGE, "qwe");
                times = 1;
            }
        };

        instance.destroy();
        temp.delete();
    }

}
