/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import mockit.FullVerifications;
import mockit.Injectable;
import mockit.Mock;
import mockit.MockUp;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.SubnodeConfiguration;
import org.apache.commons.configuration.XMLConfiguration;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class QuartzSimpleTimedEventSourceTest {

    @Injectable
    SimpleEventSource mockedEventSource;

    public QuartzSimpleTimedEventSourceTest() {
    }

    @BeforeClass
    public static void setUpClass() {
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of init method, of class QuartzSimpleTimedEventSource.
     */
    @Test
    public void testInit() {
        System.out.println("init");

        XMLConfiguration xmlConfig = new XMLConfiguration();
        String xmlConfigString = "        <EventSource>\n"
                + "            <ClassName>hu.sztaki.dsd.visionair.a2a3d.eventSources.SimpleTimedEvent</ClassName>\n"
                + "            <Id>TimedEvent1</Id>\n"
                + "            <Description>This is for timed events</Description>\n"
                + "            <Parameters>\n"
                + "                <Timeout>1</Timeout>\n"
                + "            </Parameters>\n"
                + "        </EventSource>";
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(xmlConfigString.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException ex) {
            fail("UnsupportedEncodingException");
        }
        try {
            xmlConfig.load(stream);
        } catch (ConfigurationException ex) {
            fail("ConfigurationException");
        }
        SubnodeConfiguration sconfig = new SubnodeConfiguration(xmlConfig, xmlConfig.getRoot());

        final QuartzSimpleTimedEventSource instance = new QuartzSimpleTimedEventSource(sconfig);

        new MockUp<SimpleTimedEvent>() {
            @Mock
            public void $init() {
            }

            @Mock
            EventSource getInstance(String id) {
                return instance;
            }
        };

        instance.init();

        instance.addToQueue("id", mockedEventSource);
        try {
            Thread.sleep(5000);
        } catch (InterruptedException ex) {
        }

        new FullVerifications() {
            {
                mockedEventSource.onHookEvent((Event) withNotNull());
                minTimes = 4;
                maxTimes = 6;
            }
        };
        instance.destroy();
        try {
            Thread.sleep(3000);
        } catch (InterruptedException ex) {
        }
        new FullVerifications() {
            {
                mockedEventSource.onHookEvent((Event) withNotNull());
                maxTimes = 6;
            }
        };
    }

    /**
     * Test of destroy method, of class QuartzSimpleTimedEventSource.
     */
    @Test
    public void testDestroy() {
        System.out.println("destroy");
        // Tested during testInit
    }

    /**
     * Test of trigger method, of class QuartzSimpleTimedEventSource.
     */
    @Test
    public void testTrigger() {
        System.out.println("trigger");
        // Tested during testInit
    }

}
