/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import java.util.HashMap;
import java.util.Iterator;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import mockit.FullVerifications;
import mockit.Injectable;
import mockit.NonStrictExpectations;

/**
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class SimpleEventSourceTest {

    @Injectable
    SimpleEventSource mockedEventSource;

    public SimpleEventSourceTest() {
    }

    @BeforeClass
    public static void setUpClass() {
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() {
        new NonStrictExpectations() {
            {
                Event event = null;
                mockedEventSource.onHookEvent(event);
            }
        };
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of init method, of class SimpleEventSource.
     */
    @Test
    public void testInit() {
        System.out.println("init");
        SimpleEventSource instance = new SimpleEventSource();
        assertEquals(instance.getStatus(), EventSource.STATUS_NONE);
        instance.init();
        assertEquals(instance.getStatus(), EventSource.STATUS_RUNNING);
    }

    /**
     * Test of destroy method, of class SimpleEventSource.
     */
    @Test
    public void testDestroy() {
        System.out.println("destroy");
        SimpleEventSource instance = new SimpleEventSource();
        assertEquals(instance.getStatus(), EventSource.STATUS_NONE);
        instance.init();
        instance.addToQueue("1", mockedEventSource);
        instance.addToQueue("2", mockedEventSource);
        instance.addToQueue("3", mockedEventSource);

        Iterator iterator = instance.getTriggerQueue().keySet().iterator();
        while (iterator.hasNext()) {
            ThreadNotificator tn = instance.getTriggerQueue().get(iterator.next());
            //System.out.println("check notif thread");
            assertTrue(tn.getWorkingThread().isAlive());
        }

        instance.destroy();
        assertEquals(instance.getStatus(), EventSource.STATUS_STOPPING);

        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }

        iterator = instance.getTriggerQueue().keySet().iterator();
        while (iterator.hasNext()) {
            ThreadNotificator tn = instance.getTriggerQueue().get(iterator.next());
            assertFalse(tn.getWorkingThread().isAlive());
        }
    }

    /**
     * Test of addToQueue method, of class SimpleEventSource.
     */
    @Test
    public void testAddToQueue() {
        System.out.println("addToQueue");
        String id = "newId";
        SimpleEventSource instance = new SimpleEventSource();
        instance.addToQueue(id, instance);
        HashMap<String, ThreadNotificator> triggerQueue = instance.getTriggerQueue();
        assertTrue(triggerQueue.containsKey(id));
    }

    /**
     * Test of notifyQueue method, of class SimpleEventSource.
     */
    @Test
    public void testNotifyQueue() {
        System.out.println("notifyQueue");
        final Event event = new Event();
        SimpleEventSource instance = new SimpleEventSource();
        instance.addToQueue("id", mockedEventSource);
        instance.notifyQueue(event);
        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(event);
                minTimes = 1;
                maxTimes = 1;
            }
        };
        instance.destroy();
    }

    /**
     * Performance test of notifyQueue method, of class SimpleEventSource.
     */
    @Test
    public void performancetestNotifyQueue1() {
        System.out.println("performancetestNotifyQueue1");
        final Event event = new Event();
        SimpleEventSource instance = new SimpleEventSource();
        instance.addToQueue("id", mockedEventSource);

        for (int i = 0; i < 1000000; i++) {
            instance.notifyQueue(event);
        }
        instance.destroy();
        instance.waitForDestroy();
        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(event);
                minTimes = 1000000;
                maxTimes = 1000000;
            }
        };
    }

    /**
     * Performance test of notifyQueue method, of class SimpleEventSource.
     */
    @Test
    public void performancetestNotifyQueue2() {
        System.out.println("performancetestNotifyQueue2");
        final Event event = new Event();

        SimpleEventSource instance = new SimpleEventSource();

        for (int i = 0; i < 1000; i++) {
            instance.addToQueue("id" + i, mockedEventSource);
        }

        for (int i = 0; i < 1000; i++) {
            instance.notifyQueue(event);
        }
        instance.destroy();
        instance.waitForDestroy();
        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(event);
                minTimes = 1000000;
                maxTimes = 1000000;
            }
        };
    }

    /**
     * Test of getClassName method, of class SimpleEventSource.
     */
    @Test
    public void testGetClassName() {
        System.out.println("getClassName");
        SimpleEventSource instance = new SimpleEventSource();
        String expResult = "hu.sztaki.dsd.visionair.a2a3d.eventSources.SimpleEventSource";
        String result = instance.getClassName();
        assertEquals(expResult, result);
    }

    /**
     * Test of getStatus method, of class SimpleEventSource.
     */
    @Test
    public void testGetStatus() {
        System.out.println("getStatus");
        SimpleEventSource instance = new SimpleEventSource();
        assertEquals(instance.getStatus(), EventSource.STATUS_NONE);
        instance.init();
        assertEquals(instance.getStatus(), EventSource.STATUS_RUNNING);
        instance.destroy();
        assertEquals(instance.getStatus(), EventSource.STATUS_STOPPING);
    }

    /**
     * Test of getTriggerQueue method, of class SimpleEventSource.
     */
    @Test
    public void testGetTriggerQueue() {
        System.out.println("getTriggerQueue");
        SimpleEventSource instance = new SimpleEventSource();
        assertEquals(0, instance.getTriggerQueue().size());
        instance.addToQueue("id", mockedEventSource);
        assertEquals(1, instance.getTriggerQueue().size());
        instance.destroy();
    }

    /**
     * Test of onHookEvent method, of class SimpleEventSource.
     */
    @Test
    public void testOnHookEvent() {
        System.out.println("onHookEvent");
        // Nothing to test here. The onHookEvent function is ok if testNotifyQueue() is ok.
    }

    /**
     * Test of setStatus method, of class SimpleEventSource.
     */
    @Test
    public void testSetStatus() {
        System.out.println("setStatus");
        SimpleEventSource instance = new SimpleEventSource();
        String expected = "status";
        instance.setStatus(expected);
        assertEquals(expected, instance.getStatus());
    }
}
