/*
 * Copyright 2014 MTA SZTAKI - DSD. All rights reserved.
 * MTA SZTAKI - DSD PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package hu.sztaki.dsd.visionair.a2a3d.eventSources;

import hu.sztaki.dsd.visionair.a2a3d.Event;
import mockit.FullVerifications;
import mockit.Injectable;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * @author Zoltan Toth &lt;<a
 * href="mailto:toth.zoltan@sztaki.mta.hu">toth.zoltan@sztaki.mta.hu</a>&gt; (MTA
 * SZTAKI, <a href="http://dsd.sztaki.hu">DSD</a>)
 */
public class ThreadNotificatorTest {

    @Injectable
    SimpleEventSource mockedEventSource;

    public ThreadNotificatorTest() {
    }

    @BeforeClass
    public static void setUpClass() {
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of addEvent method, of class ThreadNotificator.
     */
    @Test
    public void testAddEvent() {
        System.out.println("addEvent");
        final Event event = new Event();
        ThreadNotificator instance = new ThreadNotificator(mockedEventSource);
        assertEquals(0, instance.getEventList().size());
        instance.addEvent(event);
        instance.addEvent(event);
        assertEquals(2, instance.getEventList().size());
    }

    /**
     * Test of run method, of class ThreadNotificator.
     */
    @Test
    public void testRun() {
        System.out.println("run");
        final Event event = new Event();
        ThreadNotificator instance = new ThreadNotificator(mockedEventSource);
        assertEquals(instance.getEventList().size(), 0);
        instance.addEvent(event);
        instance.addEvent(event);
        assertEquals(2, instance.getEventList().size());
        Thread tnThread = new Thread(instance);
        tnThread.start();
        try {
            Thread.sleep(1000);
        } catch (InterruptedException ex) {
        }

        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(event);
                minTimes = 2;
                maxTimes = 2;
            }
        };
        instance.addEvent(event);
        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(event);
                minTimes = 3;
                maxTimes = 3;
            }
        };
        assertEquals(0, instance.getEventList().size());
        instance.notificatorShutDown();
        try {
            Thread.sleep(500);
        } catch (InterruptedException ex) {
        }

        assertEquals(tnThread.isAlive(), false);
    }

    /**
     * Test of run method, of class ThreadNotificator.
     */
    @Test
    public void testRunPerformanceCheck() {
        System.out.println("PerformanceCheck");
        final Event event = new Event();
        ThreadNotificator instance = new ThreadNotificator(mockedEventSource);
        Thread tnThread = new Thread(instance);
        tnThread.start();

        for (int i = 0; i < 1000000; i++) {
            instance.addEvent(event);
        }
        while (instance.getEventList().size() != 0) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException ex) {
            }
        }
        new FullVerifications() {
            {
                mockedEventSource.onHookEvent(event);
                minTimes = 1000000;
                maxTimes = 1000000;
            }
        };
        instance.notificatorShutDown();
        System.out.println("PerformanceCheck - finished");
    }

    /**
     * Test of getThread method, of class ThreadNotificator.
     */
    @Test
    public void testGetWorkingThread() {
        System.out.println("getWorkingThread");
        ThreadNotificator instance = new ThreadNotificator(mockedEventSource);
        Thread tnThread = new Thread(instance);
        assertEquals(null, instance.getWorkingThread());
        tnThread.start();
        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertNotNull(instance.getWorkingThread());
        assertTrue(instance.getWorkingThread().isAlive());
        instance.notificatorShutDown();

        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertFalse(instance.getWorkingThread().isAlive());
    }

    /**
     * Test of shutDown method, of class ThreadNotificator.
     */
    @Test
    public void testShutDown() {
        System.out.println("shutDown");
        System.out.println("getWorkingThread");
        ThreadNotificator instance = new ThreadNotificator(mockedEventSource);
        Thread tnThread = new Thread(instance);
        assertEquals(null, instance.getWorkingThread());
        tnThread.start();
        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertNotNull(instance.getWorkingThread());
        assertTrue(instance.getWorkingThread().isAlive());
        instance.notificatorShutDown();

        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
        }
        assertFalse(instance.getWorkingThread().isAlive());
    }

    /**
     * Test of getEventList method, of class ThreadNotificator.
     */
    @Test
    public void testGetEventList() {
        System.out.println("getEventList");
        final Event event = new Event();
        ThreadNotificator instance = new ThreadNotificator(mockedEventSource);
        assertEquals(instance.getEventList().size(), 0);
        instance.addEvent(event);
        assertEquals(event, instance.getEventList().getFirst());
    }

}
