/*
 * Copyright (c) 2006-2013 Rogério Liesenfeld
 * This file is subject to the terms of the MIT license (see LICENSE.txt).
 */
package mockit.internal.expectations.mocking;

import java.lang.reflect.*;

import org.jetbrains.annotations.*;

import mockit.internal.expectations.*;
import mockit.internal.state.*;
import mockit.internal.util.*;

public final class LocalFieldTypeRedefinitions extends FieldTypeRedefinitions
{
   @NotNull private final RecordAndReplayExecution execution;

   public LocalFieldTypeRedefinitions(@NotNull Object objectWithMockFields, @NotNull RecordAndReplayExecution execution)
   {
      super(objectWithMockFields);
      this.execution = execution;
   }

   public void redefineLocalFieldTypes()
   {
      redefineFieldTypes(parentObject.getClass(), false);
   }

   @Override
   protected void redefineTypeForMockField(@NotNull MockedType mockedType, @NotNull Field mockField, boolean isFinal)
   {
      TypeRedefinition typeRedefinition = new TypeRedefinition(parentObject, mockedType);

      if (isFinal) {
         typeRedefinition.redefineTypeForFinalField();

         if (mockedType.getMaxInstancesToCapture() > 0 && typeRedefinition.targetClass != TypeVariable.class) {
            execution.addMockedTypeToMatchOnInstance(typeRedefinition.targetClass);
         }

         TestRun.getExecutingTest().addFinalLocalMockField(parentObject, mockedType);
      }
      else {
         Object mock = typeRedefinition.redefineType().create();
         FieldReflection.setFieldValue(mockField, parentObject, mock);
         registerMock(mockedType, mock);
      }

      execution.addLocalMock(mockedType.declaredType, parentObject);
      addTargetClass(mockedType);

      System.out.println(
         "WARNING: Local mock field \"" + mockedType.mockId +
         "\" should be moved to the test class or converted to a parameter of the test method");
      System.out.println("  at " + new StackTrace().findPositionInTestMethod());
   }

   @Override
   public boolean captureNewInstanceForApplicableMockField(@NotNull Object mock)
   {
      return captureOfNewInstances != null && captureOfNewInstances.captureNewInstance(parentObject, mock);
   }
}
